import { Associative, KeyedMapFn } from '@collectable/core';
import { RedBlackTree, RedBlackTreeEntry, RedBlackTreeIterator } from './internals';
import { UpdateTreeCallback, UpdateTreeEntryCallback, FindOp } from './functions';
export { empty, isRedBlackTree, first, firstKey, firstValue, iterateFromFirst, last, lastKey, lastValue, iterateFromLast, size, isEmpty, thaw, isThawed, freeze, isFrozen } from './functions';
export interface RemoveFn {
    <K, V>(key: K, tree: RedBlackTree<K, V>): RedBlackTree<K, V>;
    <K, V>(key: K): (tree: RedBlackTree<K, V>) => RedBlackTree<K, V>;
}
export declare const remove: RemoveFn;
export interface SetFn {
    <K, V>(key: K, value: V, tree: RedBlackTree<K, V>): RedBlackTree<K, V>;
    <K, V>(key: K): (value: V, tree: RedBlackTree<K, V>) => RedBlackTree<K, V>;
    <K, V>(key: K, value: V): (tree: RedBlackTree<K, V>) => RedBlackTree<K, V>;
    <K, V>(key: K): (value: V) => (tree: RedBlackTree<K, V>) => RedBlackTree<K, V>;
}
export declare const set: SetFn;
export interface UpdateTreeFn {
    <K, V>(callback: UpdateTreeCallback<K, V>, tree: RedBlackTree<K, V>): RedBlackTree<K, V>;
    <K, V>(callback: UpdateTreeCallback<K, V>): (tree: RedBlackTree<K, V>) => RedBlackTree<K, V>;
}
export declare const updateTree: UpdateTreeFn;
export interface UpdateFn {
    <K, V>(callback: UpdateTreeEntryCallback<K, V | undefined>, key: K, tree: RedBlackTree<K, V>): RedBlackTree<K, V>;
    <K, V>(callback: UpdateTreeEntryCallback<K, V | undefined>): (key: K, tree: RedBlackTree<K, V>) => RedBlackTree<K, V>;
    <K, V>(callback: UpdateTreeEntryCallback<K, V | undefined>, key: K): (tree: RedBlackTree<K, V>) => RedBlackTree<K, V>;
    <K, V>(callback: UpdateTreeEntryCallback<K, V | undefined>): (key: K) => (tree: RedBlackTree<K, V>) => RedBlackTree<K, V>;
}
export declare const update: UpdateFn;
export interface GetFn {
    <K, V>(key: K, tree: RedBlackTree<K, V>): V | undefined;
    <K, V>(key: K): (tree: RedBlackTree<K, V>) => V | undefined;
}
export declare const get: GetFn;
export interface HasFn {
    <K, V>(key: K, tree: RedBlackTree<K, V>): boolean;
    <K, V>(key: K): (tree: RedBlackTree<K, V>) => boolean;
}
export declare const has: HasFn;
export interface IterateFromKeyFn {
    <K, V>(reverse: boolean, key: K, tree: RedBlackTree<K, V>): RedBlackTreeIterator<K, V>;
    <K, V>(reverse: boolean): (key: K, tree: RedBlackTree<K, V>) => RedBlackTreeIterator<K, V>;
    <K, V>(reverse: boolean, key: K): (tree: RedBlackTree<K, V>) => RedBlackTreeIterator<K, V>;
    <K, V>(reverse: boolean): (key: K) => (tree: RedBlackTree<K, V>) => RedBlackTreeIterator<K, V>;
}
export declare const iterateFromKey: IterateFromKeyFn;
export interface AtFn {
    <K, V>(index: number, tree: RedBlackTree<K, V>): RedBlackTreeEntry<K, V> | undefined;
    <K, V>(index: number): (tree: RedBlackTree<K, V>) => RedBlackTreeEntry<K, V> | undefined;
}
export declare const at: AtFn;
export interface KeyAtFn {
    <K, V>(index: number, tree: RedBlackTree<K, V>): K | undefined;
    <K, V>(index: number): (tree: RedBlackTree<K, V>) => K | undefined;
}
export declare const keyAt: KeyAtFn;
export interface ValueAtFn {
    <K, V>(index: number, tree: RedBlackTree<K, V>): V | undefined;
    <K, V>(index: number): (tree: RedBlackTree<K, V>) => V | undefined;
}
export declare const valueAt: ValueAtFn;
export interface IndexOfFn {
    <K, V>(key: K, tree: RedBlackTree<K, V>): number;
    <K, V>(key: K): (tree: RedBlackTree<K, V>) => number;
}
export declare const indexOf: IndexOfFn;
export interface IterateFromIndexFn {
    <K, V>(reverse: boolean, index: number, tree: RedBlackTree<K, V>): RedBlackTreeIterator<K, V>;
    <K, V>(reverse: boolean): (index: number, tree: RedBlackTree<K, V>) => RedBlackTreeIterator<K, V>;
    <K, V>(reverse: boolean, index: number): (tree: RedBlackTree<K, V>) => RedBlackTreeIterator<K, V>;
    <K, V>(reverse: boolean): (index: number) => (tree: RedBlackTree<K, V>) => RedBlackTreeIterator<K, V>;
}
export declare const iterateFromIndex: IterateFromIndexFn;
export interface IsEqualFn {
    <K, V>(tree: RedBlackTree<K, V>, other: RedBlackTree<K, V>): boolean;
    <K, V>(tree: RedBlackTree<K, V>): (other: RedBlackTree<K, V>) => boolean;
}
export declare const isEqual: IsEqualFn;
export interface FindFn {
    <K, V>(op: FindOp, key: K, tree: RedBlackTree<K, V>): RedBlackTreeEntry<K, V> | undefined;
    <K, V>(op: FindOp): (key: K, tree: RedBlackTree<K, V>) => RedBlackTreeEntry<K, V> | undefined;
    <K, V>(op: FindOp, key: K): (tree: RedBlackTree<K, V>) => RedBlackTreeEntry<K, V> | undefined;
    <K, V>(op: FindOp): (key: K) => (tree: RedBlackTree<K, V>) => RedBlackTreeEntry<K, V> | undefined;
}
export declare const find: FindFn;
export interface FindKeyFn {
    <K, V>(op: FindOp, key: K, tree: RedBlackTree<K, V>): K | undefined;
    <K, V>(op: FindOp): (key: K, tree: RedBlackTree<K, V>) => K | undefined;
    <K, V>(op: FindOp, key: K): (tree: RedBlackTree<K, V>) => K | undefined;
    <K, V>(op: FindOp): (key: K) => (tree: RedBlackTree<K, V>) => K | undefined;
}
export declare const findKey: FindKeyFn;
export interface FindValueFn {
    <K, V>(op: FindOp, key: K, tree: RedBlackTree<K, V>): V | undefined;
    <K, V>(op: FindOp): (key: K, tree: RedBlackTree<K, V>) => V | undefined;
    <K, V>(op: FindOp, key: K): (tree: RedBlackTree<K, V>) => V | undefined;
    <K, V>(op: FindOp): (key: K) => (tree: RedBlackTree<K, V>) => V | undefined;
}
export declare const findValue: FindValueFn;
export interface IterateFromFn {
    <K, V>(op: FindOp, reverse: boolean, key: K, tree: RedBlackTree<K, V>): RedBlackTreeIterator<K, V>;
    <K, V>(op: FindOp, reverse: boolean, key: K): (tree: RedBlackTree<K, V>) => RedBlackTreeIterator<K, V>;
    <K, V>(op: FindOp, reverse: boolean): (key: K, tree: RedBlackTree<K, V>) => RedBlackTreeIterator<K, V>;
    <K, V>(op: FindOp, reverse: boolean): (key: K) => (tree: RedBlackTree<K, V>) => RedBlackTreeIterator<K, V>;
    <K, V>(op: FindOp): (reverse: boolean, key: K, tree: RedBlackTree<K, V>) => RedBlackTreeIterator<K, V>;
    <K, V>(op: FindOp): (reverse: boolean, key: K) => (tree: RedBlackTree<K, V>) => RedBlackTreeIterator<K, V>;
    <K, V>(op: FindOp): (reverse: boolean) => (key: K, tree: RedBlackTree<K, V>) => RedBlackTreeIterator<K, V>;
    <K, V>(op: FindOp): (reverse: boolean) => (key: K) => (tree: RedBlackTree<K, V>) => RedBlackTreeIterator<K, V>;
}
export declare const iterateFrom: IterateFromFn;
export interface ArrayFromFn {
    <K, V, U>(mapper: KeyedMapFn<K, V, U>, tree: RedBlackTree<K, V>): U[];
    <K, V, U>(mapper: KeyedMapFn<K, V, U>): (tree: RedBlackTree<K, V>) => U[];
}
export declare const arrayFrom: ArrayFromFn;
export interface UnwrapFn {
    <K, V>(deep: boolean, tree: RedBlackTree<K, V>): Associative<V>;
    <K, V>(deep: boolean): (tree: RedBlackTree<K, V>) => Associative<V>;
}
export declare const unwrap: UnwrapFn;
