import { RedBlackTree, RedBlackTreeIterator } from '../internals';
/**
 * Retrieves the value associated with the specified key
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {K} key The key of the entry to retrieve
 * @param {RedBlackTree<K, V>} tree The input tree
 * @returns {(V|undefined)} The value associated with the specified key, or undefined if the key does not exist in the tree
 */
export declare function get<K, V>(key: K, tree: RedBlackTree<K, V>): V | undefined;
/**
 * Determines whether or not a given key exists in the tree
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {K} key The key to look for
 * @param {RedBlackTree<K, V>} tree The input tree
 * @returns {boolean} True if the there is an entry for the specified key, otherwise false
 */
export declare function has<K, V>(key: K, tree: RedBlackTree<K, V>): boolean;
/**
 * Creates an iterator for which the first entry has the specified index in the tree. If the key does not exist in the
 * tree, an empty iterator is returned.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {boolean} reverse If true, the iterator will iterate backward toward the first entry in the tree
 * @param {K} key The key to look for
 * @param {RedBlackTree<K, V>} tree The input tree
 * @returns {RedBlackTreeIterator<K, V>} An iterator that retrieves each successive entry in the tree, starting from the specified key
 */
export declare function iterateFromKey<K, V>(reverse: boolean, key: K, tree: RedBlackTree<K, V>): RedBlackTreeIterator<K, V>;
